/*
This file is part of libmsppath
Copyright © 2006  Mikko Rasa, Mikkosoft Productions
Distributed under the LGPL
*/
#include <dirent.h>
#include <sys/stat.h>
#include <errno.h>
#include <fnmatch.h>
#include <msp/strutils.h>
#include "path.h"
#include "utils.h"

using namespace std;

namespace Msp {
namespace Path {

/**
Fixes the case of the given path to match files / directories on the
filesystem.  Intended to be used in programs that need to interact with
emulated Windows programs.
*/
Path fix_case(const Path &path)
{
	bool found=true;
	Path result;
	for(Path::iterator i=path.begin(); i!=path.end(); ++i)
	{
		if(!found || *i=="/")
			result/=*i;
		else
		{
			list<string> files;
			if(result.size())
				files=list_files(result);
			else
				files=list_files(".");

			found=false;
			for(list<string>::iterator j=files.begin(); (j!=files.end() && !found); ++j)
				if(!strcasecmp(*j,*i))
				{
					result/=*j;
					found=true;
				}

			if(!found)
				result/=*i;
		}
	}

	return result;
}

int mkpath(const Path &path, int mode)
{
	Path p;
	for(Path::iterator i=path.begin(); i!=path.end(); ++i)
	{
		p/=*i;
#ifdef WIN32
		if(p.size()==1 && is_windows_drive(*i)) continue;
#endif
		struct stat st;
		int err=stat(p.str().c_str(),&st);
		if(err==0)
		{
			if(!S_ISDIR(st.st_mode))
			{
				errno=EEXIST;
				return -1;
			}
			continue;
		}
		else if(errno!=ENOENT)
			return -1;
		else
		{
#ifdef WIN32
			// The win32 version of this function doesn't take the mode argument.  Go figure.
			err=mkdir(p.str().c_str());
#else
			err=mkdir(p.str().c_str(),mode);
#endif
			if(err==-1) return -1;
		}
	}

	return 0;
}

int rmdir(const Path &path, bool recursive)
{
	if(recursive)
	{
		list<string> files=list_files(path);
		for(list<string>::iterator i=files.begin(); i!=files.end(); ++i)
		{
			Path p=path/ *i;
			struct stat st;
			stat(p.str().c_str(),&st);
			int err=0;
			if(S_ISDIR(st.st_mode))
				err=rmdir(p,true);
			else
				err=unlink(p.str().c_str());
			if(err) return err;
		}
	}
	return rmdir(path.str().c_str());
}

/**
Lists all files in a directory except the implied . and .. entries.
*/
list<string> list_files(const Path &path)
{
	list<string> result;
	DIR *dir=opendir(path.str().c_str());
	if(dir)
	{
		while(dirent *de=readdir(dir))
		{
			const char *fn=de->d_name;
			if(fn[0]=='.' && (fn[1]==0 || (fn[1]=='.' && fn[2]==0))) continue;
			result.push_back(fn);
		}
		closedir(dir);
	}
	return result;
}

bool exists(const Path &path)
{
	struct stat st;
	return !stat(path.str().c_str(),&st);
}

Filename splitext(const string &fn)
{
	Filename result;
	unsigned dot=fn.rfind('.');
	result.base=fn.substr(0,dot);
	if(dot!=string::npos)
		result.ext=fn.substr(dot);
	return result;
}

int fnmatch(const string &pat, const Path &fn)
{
	return ::fnmatch(pat.c_str(), fn.str().c_str(), FNM_PATHNAME);
}

} // namespace Path
} // namespace Msp
